import { ViewChild, ElementRef, Input, Output, EventEmitter, TemplateRef, } from '@angular/core';
import { readFileAsDataURL } from '../utils/dom-utils';
import { formatFileSize, formatString } from '../utils/lange-utils';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import { of } from 'rxjs/observable/of';
import { forkJoin } from 'rxjs/observable/forkJoin';
import { noop } from '../utils';
var FileUpload = /** @class */ (function () {
    function FileUpload(rebirthNGConfig, renderer, http, changeDetectorRef) {
        this.rebirthNGConfig = rebirthNGConfig;
        this.renderer = renderer;
        this.http = http;
        this.changeDetectorRef = changeDetectorRef;
        this.multiple = true;
        this.showErrors = true;
        this.canRetry = true;
        this.selectFilesChange = new EventEmitter();
        this.fileUploadStart = new EventEmitter();
        this.fileUploadCompleted = new EventEmitter();
        this.fileUploadSuccess = new EventEmitter();
        this.fileUploadError = new EventEmitter();
        this.removeFiles = new EventEmitter();
        this.uploadFilesChange = new EventEmitter();
        this.uploadFiles = [];
        this.selectFiles = [];
        this.errors = [];
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.fileSizeErrorMessage = this.rebirthNGConfig.fileUpload.fileSizeErrorMessage;
        this.fileTypeErrorMessage = this.rebirthNGConfig.fileUpload.fileTypeErrorMessage;
        this.uploadParamName = this.rebirthNGConfig.fileUpload.uploadParamName;
        this.previewWidth = this.rebirthNGConfig.fileUpload.previewWidth;
        this.previewHeight = this.rebirthNGConfig.fileUpload.previewHeight;
        this.imgPreview = this.rebirthNGConfig.fileUpload.imgPreview;
        this.chooseButton = this.rebirthNGConfig.fileUpload.chooseButton;
        this.uploadButton = this.rebirthNGConfig.fileUpload.uploadButton;
        this.cancelButton = this.rebirthNGConfig.fileUpload.cancelButton;
        this.plusIcon = this.rebirthNGConfig.fileUpload.plusIcon;
        this.uploadIcon = this.rebirthNGConfig.fileUpload.uploadIcon;
        this.loadingIcon = this.rebirthNGConfig.fileUpload.loadingIcon;
        this.removeIcon = this.rebirthNGConfig.fileUpload.removeIcon;
        this.showErrors = this.rebirthNGConfig.fileUpload.showErrors;
        this.transformResponseUrl = this.rebirthNGConfig.fileUpload.transformResponseUrl;
    }
    FileUpload.prototype.ngAfterViewInit = function () {
        if (this.accept) {
            this.renderer.setProperty(this.fileInput.nativeElement, 'accept', this.accept);
        }
        this.renderer.setProperty(this.fileInput.nativeElement, 'multiple', this.multiple);
    };
    FileUpload.prototype.writeValue = function (value) {
        this.uploadFiles = value || [];
        this.changeDetectorRef.markForCheck();
    };
    FileUpload.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    FileUpload.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    FileUpload.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    FileUpload.prototype.isMoreThanMaxItems = function () {
        var fileSize = this.getFileCount();
        if (!this.multiple) {
            return fileSize >= 1;
        }
        return this.maxItems && (fileSize >= this.maxItems);
    };
    FileUpload.prototype.clearErrors = function () {
        this.errors = [];
    };
    FileUpload.prototype.onDropFiles = function ($event) {
        if (this.disabled) {
            return;
        }
        $event.stopPropagation();
        $event.preventDefault();
        this.clearErrors();
        var files = $event.dataTransfer.files;
        if (files && files.length) {
            this.handleFileChoose(files);
        }
    };
    FileUpload.prototype.addNewFile = function ($event) {
        if (this.disabled) {
            return;
        }
        $event.stopPropagation();
        if (!this.isMoreThanMaxItems()) {
            this.fileInput.nativeElement.value = null;
            this.fileInput.nativeElement.click(); // simulate file input event
        }
    };
    FileUpload.prototype.newFileChoose = function (fileInput) {
        this.clearErrors();
        if (fileInput.files && fileInput.files.length) {
            this.handleFileChoose(fileInput.files);
        }
    };
    FileUpload.prototype.removeAllSelectedFiles = function () {
        this.onTouched();
        var files = this.selectFiles;
        this.selectFiles = [];
        this.clearErrors();
        this.removeFiles.emit(files);
    };
    FileUpload.prototype.onRemoveFile = function (fileItem) {
        if (this.disabled) {
            return;
        }
        this.onTouched();
        this.selectFiles = this.selectFiles.filter(function (item) { return item !== fileItem; });
        this.removeFiles.emit([fileItem]);
    };
    FileUpload.prototype.onRemoveUploadFile = function (fileItem) {
        if (this.disabled) {
            return;
        }
        this.onTouched();
        this.onUploadFileChange(this.uploadFiles.filter(function (item) { return item !== fileItem; }));
        this.removeFiles.emit([fileItem]);
    };
    FileUpload.prototype.uploadAllFiles = function () {
        this.clearErrors();
        this.httpUploadAllFile(this.selectFiles);
    };
    FileUpload.prototype.httpUploadAllFile = function (files) {
        var _this = this;
        this.isUploading = true;
        this.fileUploadStart.emit(files);
        var subscriptions = files.map(function (fileItem) { return _this.httpUploadFile(fileItem); });
        forkJoin(subscriptions)
            .subscribe(noop, noop, function () {
            _this.isUploading = false;
            _this.fileUploadCompleted.emit(files);
            _this.onTouched();
        });
    };
    FileUpload.prototype.httpUploadFile = function (fileItem) {
        var _this = this;
        var formData = new FormData();
        formData.append(this.uploadParamName, fileItem.file);
        return this.http.post(this.uploadUrl, formData, this.uploadRequestOptions)
            .map(function (res) { return _this.onFileUploadSuccess(fileItem, res); })
            .catch(function (error) { return _this.onFileUploadError(fileItem, error); });
    };
    FileUpload.prototype.onFileUploadSuccess = function (fileItem, res) {
        fileItem.uploadResponse = res;
        fileItem.url = this.transformResponseUrl(res) || fileItem.url;
        this.fileUploadSuccess.emit(fileItem);
        this.selectFiles = this.selectFiles.filter(function (item) { return item !== fileItem; });
        this.onUploadFileChange((this.uploadFiles || []).concat([fileItem]));
        this.changeDetectorRef.markForCheck();
        return of({ result: res, success: true });
    };
    FileUpload.prototype.onUploadFileChange = function (uploadFiles) {
        this.uploadFiles = uploadFiles;
        this.uploadFilesChange.emit(uploadFiles);
        this.onChange(uploadFiles);
    };
    FileUpload.prototype.onFileUploadError = function (fileItem, error) {
        this.errors.push(fileItem.name + ": " + (error.error || error.statusText));
        this.fileUploadError.emit({
            name: fileItem.name,
            displaySize: fileItem.displaySize,
            url: fileItem.url,
            file: fileItem.file,
            uploadResponse: error
        });
        if (!this.canRetry) {
            this.onRemoveFile(fileItem);
        }
        this.changeDetectorRef.markForCheck();
        return of({ error: error, success: false });
    };
    FileUpload.prototype.handleFileChoose = function (uploadFiles) {
        var _this = this;
        var files = this.validFiles(Array.from(uploadFiles));
        this.mapFileModel(files)
            .then(function (fileModels) {
            _this.selectFiles = _this.selectFiles.concat(fileModels);
            _this.selectFilesChange.emit(_this.selectFiles);
            _this.changeDetectorRef.markForCheck();
            return fileModels || [];
        })
            .then(function (fileModels) {
            if (_this.autoUpload) {
                return _this.httpUploadAllFile(fileModels);
            }
        });
    };
    FileUpload.prototype.validFiles = function (files) {
        var _this = this;
        var fileCount = this.getFileCount();
        var size = this.multiple ? this.maxItems : 1;
        if (size && (fileCount + files.length > size)) {
            files = files.slice(0, size - fileCount);
        }
        return files.filter(function (file) {
            return _this.validFile(file);
        });
    };
    FileUpload.prototype.validFile = function (file) {
        var errors = [];
        if (this.maxFileSize && file.size > this.maxFileSize) {
            errors.push(formatString(this.fileSizeErrorMessage, file.name, formatFileSize(this.maxFileSize)));
        }
        if (this.accept && !this.validFileType(file)) {
            errors.push(formatString(this.fileTypeErrorMessage, file.name, this.accept));
        }
        (_a = this.errors).push.apply(_a, errors);
        if (this.errors.length) {
            this.fileUploadError.emit({
                name: file.name,
                file: file,
                uploadResponse: this.errors
            });
        }
        return !errors.length;
        var _a;
    };
    FileUpload.prototype.getFileCount = function () {
        return ((this.selectFiles || []).length + (this.uploadFiles || []).length);
    };
    FileUpload.prototype.validFileType = function (file) {
        return this.accept.split(',').some(function (type) {
            return new RegExp("^" + type.replace(/\*/g, '.*') + "$").test(file.type);
        });
    };
    FileUpload.prototype.mapFileModel = function (files) {
        return Promise.all(files.map(function (file) {
            return readFileAsDataURL(file)
                .then(function (url) { return ({ url: url, name: file.name, file: file, displaySize: formatFileSize(file.size) }); });
        }));
    };
    FileUpload.propDecorators = {
        "accept": [{ type: Input },],
        "multiple": [{ type: Input },],
        "autoUpload": [{ type: Input },],
        "showErrors": [{ type: Input },],
        "maxItems": [{ type: Input },],
        "maxFileSize": [{ type: Input },],
        "uploadUrl": [{ type: Input },],
        "uploadParamName": [{ type: Input },],
        "uploadRequestOptions": [{ type: Input },],
        "imgPreview": [{ type: Input },],
        "previewWidth": [{ type: Input },],
        "previewHeight": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "fileSizeErrorMessage": [{ type: Input },],
        "fileTypeErrorMessage": [{ type: Input },],
        "chooseButton": [{ type: Input },],
        "uploadButton": [{ type: Input },],
        "cancelButton": [{ type: Input },],
        "plusIcon": [{ type: Input },],
        "uploadIcon": [{ type: Input },],
        "loadingIcon": [{ type: Input },],
        "removeIcon": [{ type: Input },],
        "canRetry": [{ type: Input },],
        "toolbarTemplate": [{ type: Input },],
        "previewTemplate": [{ type: Input },],
        "disabled": [{ type: Input },],
        "selectFilesChange": [{ type: Output },],
        "fileUploadStart": [{ type: Output },],
        "fileUploadCompleted": [{ type: Output },],
        "fileUploadSuccess": [{ type: Output },],
        "fileUploadError": [{ type: Output },],
        "removeFiles": [{ type: Output },],
        "uploadFilesChange": [{ type: Output },],
        "uploadFiles": [{ type: Input },],
        "transformResponseUrl": [{ type: Input },],
        "fileInput": [{ type: ViewChild, args: ['file',] },],
    };
    return FileUpload;
}());
export { FileUpload };
//# sourceMappingURL=file-upload.js.map