import { Component, ChangeDetectionStrategy, Input, Output, EventEmitter, HostListener, ElementRef, TemplateRef } from '@angular/core';
import { WindowRef } from '../window-ref/window-ref.service';
var InfiniteScrollComponent = /** @class */ (function () {
    function InfiniteScrollComponent(windowRef, elementRef) {
        this.windowRef = windowRef;
        this.elementRef = elementRef;
        this.bufferPx = 40;
        this.scrollDelay = 300;
        this.loadMore = new EventEmitter();
        this.scrollStream = new EventEmitter();
    }
    InfiniteScrollComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.setupScrollEvent();
        setTimeout(function (_) { return _this.onWinScroll(null); });
    };
    InfiniteScrollComponent.prototype.loadFinish = function (complete) {
        var _this = this;
        if (complete) {
            return this.unSubscription();
        }
        setTimeout(function (_) { return _this.onWinScroll(null); }, this.scrollDelay);
    };
    InfiniteScrollComponent.prototype.setupScrollEvent = function () {
        var _this = this;
        if (!this.subscription) {
            this.subscription = this.registerOnScrollStream(this.scrollStream)
                .subscribe(function (_) { return _this.onScrollChange(); });
        }
    };
    InfiniteScrollComponent.prototype.onScrollChange = function () {
        var winHeight = this.windowRef.innerHeight;
        var clientRect = this.windowRef.getBoundingClientRect(this.elementRef);
        if (clientRect && winHeight - clientRect.top >= this.bufferPx) {
            this.loadMore.emit(this);
        }
    };
    InfiniteScrollComponent.prototype.onWinScroll = function ($event) {
        this.scrollStream.emit($event);
    };
    InfiniteScrollComponent.prototype.ngOnDestroy = function () {
        this.unSubscription();
    };
    InfiniteScrollComponent.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    InfiniteScrollComponent.prototype.registerOnScrollStream = function (scrollStream) {
        return scrollStream
            .debounceTime(this.scrollDelay)
            .distinctUntilChanged();
    };
    InfiniteScrollComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-infinite-scroll',
                    template: "\n    <ng-template #defaultTemplate let-spinner=\"\"><div class=\"spinner\"><div class=\"bounce bounce1\"></div><div class=\"bounce bounce2\"></div><div class=\"bounce bounce3\"></div></div></ng-template><ng-template [ngTemplateOutlet]=\"spinnerTemplate || defaultTemplate\" [ngTemplateOutletContext]=\"{ $implicit: this }\"></ng-template>\n  ",
                    styles: ["\n    .spinner{margin:10px auto 0;width:150px;text-align:center}.spinner .bounce{width:24px;height:24px;background-color:#e3c381;border-radius:100%;display:inline-block;-webkit-animation:bouncedelay 1.4s infinite ease-in-out;animation:bouncedelay 1.4s infinite ease-in-out;-webkit-animation-fill-mode:both;animation-fill-mode:both}.spinner .bounce1{-webkit-animation-delay:-.32s;animation-delay:-.32s}.spinner .bounce2{-webkit-animation-delay:-.16s;animation-delay:-.16s}@-webkit-keyframes bouncedelay{0%,100%,80%{-webkit-transform:scale(0)}40%{-webkit-transform:scale(1)}}@keyframes bouncedelay{0%,100%,80%{transform:scale(0);-webkit-transform:scale(0)}40%{transform:scale(1);-webkit-transform:scale(1)}}\n  "],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'infiniteScroll'
                },] },
    ];
    /** @nocollapse */
    InfiniteScrollComponent.ctorParameters = function () { return [
        { type: WindowRef, },
        { type: ElementRef, },
    ]; };
    InfiniteScrollComponent.propDecorators = {
        "total": [{ type: Input },],
        "bufferPx": [{ type: Input },],
        "scrollDelay": [{ type: Input },],
        "spinnerTemplate": [{ type: Input },],
        "loadMore": [{ type: Output },],
        "onWinScroll": [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
    };
    return InfiniteScrollComponent;
}());
export { InfiniteScrollComponent };
//# sourceMappingURL=infinite-scroll.component.js.map