import { Component, Input, ViewChild, HostListener, EventEmitter, ElementRef, Output } from '@angular/core';
import { ModalContentComponent } from './modal-content.component';
import { ModalDismissReasons } from './modal-dismiss-reasons.model';
import { trigger, state, style, transition, animate } from '@angular/animations';
import 'rxjs/add/operator/filter';
var ModalWindowComponent = /** @class */ (function () {
    function ModalWindowComponent(elementRef) {
        this.elementRef = elementRef;
        this.isOpen = false;
        this.instanceCount = 0;
        this.animationDone = new EventEmitter();
    }
    ModalWindowComponent.prototype.open = function () {
        var _this = this;
        this.isOpen = true;
        if (this.modalOptions.animation) {
            this.animateState = 'in';
        }
        else {
            setTimeout(function () { return _this.onAnimationDone({ toState: 'in' }); });
        }
        return this.animationDone
            .filter(function (event) { return event.toState === 'in'; });
    };
    ModalWindowComponent.prototype.close = function () {
        var _this = this;
        this.isOpen = false;
        if (this.modalOptions.animation) {
            this.animateState = 'void';
        }
        else {
            setTimeout(function () { return _this.onAnimationDone({ toState: 'void' }); });
        }
        return this.animationDone
            .filter(function (event) { return event.toState === 'void'; });
    };
    ModalWindowComponent.prototype.onAnimationStart = function ($event) {
        this.isInAnimationDone = true;
    };
    ModalWindowComponent.prototype.onAnimationDone = function ($event) {
        this.isInAnimationDone = false;
        this.animationDone.emit($event);
    };
    ModalWindowComponent.prototype.onBackdropClick = function ($event) {
        if (!this.modalOptions.modal && !this.isInAnimationDone && this.elementRef.nativeElement === $event.target) {
            this.dismiss.error(ModalDismissReasons.BACKDROP_CLICK);
        }
    };
    ModalWindowComponent.prototype.onEscKeyUp = function ($event) {
        if (this.modalOptions.keyboard !== false) {
            this.dismiss.error(ModalDismissReasons.ESC_KEY);
        }
    };
    ModalWindowComponent.prototype.addContent = function (options, dismiss) {
        this.modalOptions = options;
        this.dismiss = dismiss;
        this.modalContent.addContent(options, this.dismiss);
        return this.dismiss;
    };
    ModalWindowComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-modal-window',
                    template: "\n    <div class=\"modal-dialog {{(modalOptions.size ? ' modal-' + modalOptions.size : '')}}\" role=\"document\"><div class=\"modal-content\"><re-modal-content #modalContent></re-modal-content></div></div>\n  ",
                    host: {
                        '[@flyInOut]': 'animateState',
                        '(@flyInOut.start)': 'onAnimationStart($event)',
                        '(@flyInOut.done)': 'onAnimationDone($event)'
                    },
                    animations: [
                        trigger('flyInOut', [
                            state('void', style({ top: '-100%', opacity: 0 })),
                            state('in', style({ top: '10%', opacity: 1 })),
                            transition('void => in', animate('0.2s ease-in-out')),
                            transition('in => void', animate('0.2s ease-in-out')),
                        ])
                    ]
                },] },
    ];
    /** @nocollapse */
    ModalWindowComponent.ctorParameters = function () { return [
        { type: ElementRef, },
    ]; };
    ModalWindowComponent.propDecorators = {
        "isOpen": [{ type: Input },],
        "instanceCount": [{ type: Input },],
        "animationDone": [{ type: Output },],
        "modalContent": [{ type: ViewChild, args: [ModalContentComponent,] },],
        "onBackdropClick": [{ type: HostListener, args: ['click', ['$event'],] },],
        "onEscKeyUp": [{ type: HostListener, args: ['keyup.esc', ['$event'],] },],
    };
    return ModalWindowComponent;
}());
export { ModalWindowComponent };
//# sourceMappingURL=modal-window.component.js.map