import { Component, Input, EventEmitter, ChangeDetectionStrategy, forwardRef, ChangeDetectorRef } from '@angular/core';
import { RebirthNGConfig } from '../rebirth-ng.config';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var RatingComponent = /** @class */ (function () {
    function RatingComponent(rebirthNGConfig, changeDetectorRef) {
        this.rebirthNGConfig = rebirthNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.value = 0;
        this.disabled = false;
        this.valueChange = new EventEmitter();
        this.statValue = 0;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.icons = rebirthNGConfig.rating.icons;
        this.ratingRanges = this.fillRatingRange(rebirthNGConfig.rating.max);
    }
    Object.defineProperty(RatingComponent.prototype, "max", {
        set: function (value) {
            this.ratingRanges = this.fillRatingRange(value);
        },
        enumerable: true,
        configurable: true
    });
    RatingComponent.prototype.setRatingValue = function (value) {
        this.statValue = this.value = value;
    };
    RatingComponent.prototype.writeValue = function (obj) {
        this.setRatingValue(obj);
        this.changeDetectorRef.markForCheck();
    };
    RatingComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    RatingComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    RatingComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    RatingComponent.prototype.rate = function (value) {
        if (!this.disabled && this.value !== value) {
            this.onTouched();
            this.setRatingValue(value);
            this.valueChange.emit(this.value);
            this.onChange(this.value);
        }
    };
    RatingComponent.prototype.enter = function (value) {
        if (!this.disabled) {
            this.onTouched();
            this.statValue = value;
        }
    };
    RatingComponent.prototype.reset = function () {
        this.statValue = this.value;
    };
    RatingComponent.prototype.fillRatingRange = function (range) {
        return Array(range).fill(0).map(function (_, i) { return i + 1; });
    };
    RatingComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-rating',
                    template: "\n    <span aria-labelledby=\"default-rating\" class=\"rating {{cssClass}}\"><span class=\"slider\" (mouseleave)=\"reset()\" tabindex=\"0\" role=\"slider\" [attr.aria-valuemin]=\"0\" [attr.aria-valuemax]=\"ratingRanges.length\" [attr.aria-valuenow]=\"statValue\" [attr.aria-valuetext]=\"statValue\"><ng-container *ngFor=\"let item of ratingRanges\"><span class=\"sr-only\">({{statValue >= item? \"*\": \"\"}} )</span> <i (mouseenter)=\"enter(item)\" (click)=\"rate(item)\" class=\"{{ statValue < item ? icons.stateOff: icons.stateOn}}\" [attr.title]=\"item\"></i></ng-container></span></span>\n  ",
                    styles: ["\n    .slider,.slider:focus{outline:0}\n  "],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'rating',
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return RatingComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    RatingComponent.ctorParameters = function () { return [
        { type: RebirthNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    RatingComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "cssClass": [{ type: Input },],
        "icons": [{ type: Input },],
        "max": [{ type: Input },],
    };
    return RatingComponent;
}());
export { RatingComponent };
//# sourceMappingURL=rating.component.js.map