import { Component, Input, Output, forwardRef, EventEmitter, HostListener, ElementRef, TemplateRef, ChangeDetectorRef, ChangeDetectionStrategy } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { RebirthNGConfig } from '../rebirth-ng.config';
import { trigger, state, transition, animate, style } from '@angular/animations';
var SelectComponent = /** @class */ (function () {
    function SelectComponent(changeDetectorRef, elementRef, rebirthNgConfig) {
        this.changeDetectorRef = changeDetectorRef;
        this.elementRef = elementRef;
        this.direction = 'down';
        this.onSelect = new EventEmitter();
        this.arrowState = 'down';
        this.activeIndex = 0;
        this.isPopup = false;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.iconDown = rebirthNgConfig.select.iconDown;
        this.formatter = rebirthNgConfig.select.formatter;
    }
    Object.defineProperty(SelectComponent.prototype, "groupOptions", {
        set: function (group) {
            var _this = this;
            if (group && this._groupOptions !== group) {
                this._options = group.reduce(function (opts, item) {
                    if (item.options && item.options.length) {
                        opts.push.apply(opts, [{ group: item }].concat(item.options));
                    }
                    if (_this.selectedItem) {
                        setTimeout(function () { return _this.changeValue(null); });
                    }
                    return opts;
                }, []);
            }
            this._groupOptions = group;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SelectComponent.prototype, "options", {
        get: function () {
            return this._options;
        },
        set: function (options) {
            var _this = this;
            if (options !== this._options) {
                this._options = options;
                if (this.selectedItem) {
                    setTimeout(function () { return _this.changeValue(null); });
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    SelectComponent.prototype.ngOnInit = function () {
        this.arrowState = this.direction;
    };
    SelectComponent.prototype.writeValue = function (value) {
        this.changeValue(value);
    };
    SelectComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SelectComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SelectComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    SelectComponent.prototype.onPopupToggle = function (isPopup) {
        if (!this.disabled) {
            if (isPopup === undefined) {
                isPopup = !this.isPopup;
            }
            this.isPopup = isPopup;
            var status_1 = { false: 'down', true: 'up' };
            if (this.direction === 'up') {
                status_1 = { false: 'up', true: 'down' };
            }
            this.arrowState = status_1[isPopup.toString()];
            this.updateActiveIndex();
        }
    };
    SelectComponent.prototype.onSelectedChange = function (value) {
        this.onTouched();
        this.changeValue(value);
        this.onPopupToggle(false);
        this.onSelect.emit(value);
    };
    SelectComponent.prototype.onActiveIndexChange = function (index) {
        this.activeIndex = (index + this._options.length) % this._options.length;
        return this.activeIndex;
    };
    SelectComponent.prototype.onEscKeyup = function ($event) {
        this.onPopupToggle(false);
    };
    SelectComponent.prototype.onEnterKeyDown = function ($event) {
        if (this.isPopup) {
            $event.preventDefault();
            $event.stopPropagation();
            this.onSelectedChange(this._options[this.activeIndex]);
        }
    };
    SelectComponent.prototype.onArrowUpKeyDown = function ($event) {
        if (this.isPopup) {
            $event.preventDefault();
            $event.stopPropagation();
            var index = this.onActiveIndexChange(this.activeIndex - 1);
            if (this.isGroupItem(this._options[index])) {
                this.onActiveIndexChange(this.activeIndex - 1);
            }
        }
    };
    SelectComponent.prototype.onArrowDownKeyDown = function ($event) {
        if (this.isPopup) {
            $event.preventDefault();
            $event.stopPropagation();
            var index = this.onActiveIndexChange(this.activeIndex + 1);
            if (this.isGroupItem(this._options[index])) {
                this.onActiveIndexChange(this.activeIndex + 1);
            }
        }
    };
    SelectComponent.prototype.onDocumentClick = function ($event) {
        var hostElement = this.elementRef.nativeElement;
        if (!hostElement.contains($event.target)) {
            this.onPopupToggle(false);
        }
    };
    SelectComponent.prototype.isGroupItem = function (item) {
        return item.group;
    };
    SelectComponent.prototype.changeValue = function (value) {
        this.selectedItem = value;
        this.selectedText = value ? this.formatter(value) : '';
        this.updateActiveIndex();
        this.onChange(value);
        this.changeDetectorRef.markForCheck();
    };
    SelectComponent.prototype.updateActiveIndex = function () {
        if (this._options && this.selectedItem) {
            this.activeIndex = this._options.indexOf(this.selectedItem);
        }
    };
    SelectComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-select',
                    template: "\n    <ng-template #defaultItemTemplate let-formatter=\"formatter\" let-item=\"item\">{{formatter(item)}}</ng-template><div class=\"drop{{direction}} select-container\"><div class=\"input-group select-panel\" [ngClass]=\"{disabled: disabled}\" (click)=\"onPopupToggle()\"><input class=\"form-control select-control\" [disabled]=\"disabled\" readonly=\"readonly\" unselectable=\"on\" [placeholder]=\"placeholder\" [(ngModel)]=\"selectedText\" (ngModelChange)=\"onSelectedChange($event)\"><div class=\"input-group-addon\" [ngClass]=\"{disabled: disabled}\"><i class=\"select-down-icon {{iconDown}}\" [@arrowState]=\"arrowState\"></i></div></div><div class=\"dropdown-menu\" [style.display]=\"isPopup ? 'block' : 'none'\"><ul class=\"list-unstyled popup-list\"><ng-container *ngFor=\"let item of options; let $index = index;\"><li *ngIf=\"item.group\" class=\"dropdown-header\">{{item.group?.group}}</li><li *ngIf=\"!item.group\" class=\"dropdown-item\" [ngClass]=\"{'item-hoverd bg-primary': $index == activeIndex, selected: selectedItem == item}\" (click)=\"onSelectedChange(item)\" (mouseenter)=\"onActiveIndexChange($index)\"><ng-template [ngTemplateOutlet]=\"itemTemplate || defaultItemTemplate\" [ngTemplateOutletContext]=\"{ formatter: formatter, selectedItem:selectedItem, selectedText:selectedText, options:options, item:item, $index:$index}\"></ng-template></li></ng-container></ul></div></div>\n  ",
                    styles: ["\n    :host{display:block}.select-container{position:relative;border-bottom:1px solid #ddd}.select-control:not([disabled]),.select-panel{cursor:pointer;background:#fff}.select-panel.disabled{background:#eee}.input-group .select-control{border:none}.input-group-addon{border:none}.input-group-addon:not(.disabled){background:#fff}.input-group-addon .select-down-icon{display:block}::ng-deep .dropdown-menu{border:none;padding:0;width:100%}::ng-deep .dropdown-menu .popup-list{margin-bottom:0;max-height:250px;overflow:auto}::ng-deep .dropdown-menu .dropdown-header{padding:3px .5rem}::ng-deep .dropdown-menu .dropdown-item{cursor:pointer;padding:8px 1rem;white-space:normal!important}::ng-deep .dropdown-menu .dropdown-item.selected{font-weight:700}\n  "],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'select',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return SelectComponent; }),
                            multi: true
                        }
                    ],
                    animations: [
                        trigger('arrowState', [
                            state('down', style({
                                transform: 'rotate(0deg)'
                            })),
                            state('up', style({
                                transform: 'rotate(180deg)'
                            })),
                            transition('down => up', animate('200ms ease-in')),
                            transition('up => down', animate('200ms ease-out'))
                        ])
                    ]
                },] },
    ];
    /** @nocollapse */
    SelectComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: ElementRef, },
        { type: RebirthNGConfig, },
    ]; };
    SelectComponent.propDecorators = {
        "disabled": [{ type: Input },],
        "placeholder": [{ type: Input },],
        "iconDown": [{ type: Input },],
        "direction": [{ type: Input },],
        "itemTemplate": [{ type: Input },],
        "formatter": [{ type: Input },],
        "onSelect": [{ type: Output },],
        "groupOptions": [{ type: Input },],
        "options": [{ type: Input },],
        "onEscKeyup": [{ type: HostListener, args: ['keydown.esc', ['$event'],] },],
        "onEnterKeyDown": [{ type: HostListener, args: ['keydown.Enter', ['$event'],] },],
        "onArrowUpKeyDown": [{ type: HostListener, args: ['keydown.ArrowUp', ['$event'],] },],
        "onArrowDownKeyDown": [{ type: HostListener, args: ['keydown.ArrowDown', ['$event'],] },],
        "onDocumentClick": [{ type: HostListener, args: ['document:click', ['$event'],] },],
    };
    return SelectComponent;
}());
export { SelectComponent };
//# sourceMappingURL=select.component.js.map