import { Component, ChangeDetectorRef, ChangeDetectionStrategy, forwardRef, ViewChild, ElementRef, Input, Output, EventEmitter, HostListener } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import 'rxjs/add/observable/fromEvent';
import 'rxjs/add/operator/take';
import { Subject } from 'rxjs/Subject';
import { RebirthNGConfig } from '../rebirth-ng.config';
import { DocumentRef, WindowRef } from '../window-ref';
var SliderComponent = /** @class */ (function () {
    function SliderComponent(changeDetectorRef, rebirthNGConfig, documentRef, windowRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.rebirthNGConfig = rebirthNGConfig;
        this.documentRef = documentRef;
        this.windowRef = windowRef;
        this.onChange = new EventEmitter();
        this.handleOffset = 0;
        this.value = 0;
        this.complete$ = new Subject();
        this.onValueChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.max = rebirthNGConfig.slider.max;
        this.min = rebirthNGConfig.slider.min;
    }
    SliderComponent.prototype.ngOnInit = function () {
        this.offset = this.max - this.min;
    };
    SliderComponent.prototype.ngOnDestroy = function () {
        this.complete$.complete();
    };
    SliderComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.stepWidth = this.windowRef.getOffsetWidth(this.slider) / this.offset;
        var mousedown$ = Observable.fromEvent(this.dot.nativeElement, 'mousedown');
        var mousemove$ = Observable.fromEvent(this.documentRef.document, 'mousemove');
        var mouseup$ = Observable.fromEvent(this.documentRef.document, 'mouseup');
        mousedown$
            .switchMap(function () {
            _this.onTouched();
            return mousemove$.takeUntil(mouseup$);
        })
            .takeUntil(this.complete$)
            .subscribe(function (event) {
            event.preventDefault();
            _this.calcValueWhenMove(event);
            _this.changeDetectorRef.markForCheck();
        });
        mousedown$
            .switchMap(function () {
            _this.onTouched();
            return mouseup$.take(1);
        })
            .takeUntil(this.complete$)
            .subscribe(function (event) {
            event.preventDefault();
            _this.calcValueWhenMove(event);
            _this.calcHandleOffset();
            _this.changeDetectorRef.markForCheck();
        });
    };
    SliderComponent.prototype.onResize = function () {
        this.stepWidth = this.windowRef.getOffsetWidth(this.slider) / this.offset;
        this.calcHandleOffset();
    };
    SliderComponent.prototype.onSliderClick = function (event) {
        this.calcValueWhenMove(event);
    };
    SliderComponent.prototype.onOperateValue = function (value) {
        this.onTouched();
        this.operateValue(value);
    };
    SliderComponent.prototype.operateValue = function (value) {
        this.changeValue(value);
        this.calcHandleOffset();
    };
    SliderComponent.prototype.calcHandleOffset = function () {
        this.handleOffset = this.value * this.stepWidth;
    };
    SliderComponent.prototype.calcValueWhenMove = function (event) {
        var rect = this.windowRef.getBoundingClientRect(this.slider); // IE >= 9
        var mouseRelativeSliderOffsetX = event.clientX - rect.left;
        var fixedOffsetX = Math.max(Math.min(mouseRelativeSliderOffsetX, rect.width), 0);
        this.handleOffset = fixedOffsetX;
        this.changeValue(Math.round(fixedOffsetX / this.stepWidth));
    };
    SliderComponent.prototype.changeValue = function (newValue) {
        var fixedValue = Math.max(Math.min(newValue, this.max), 0);
        if (fixedValue === this.value) {
            return;
        }
        this.value = fixedValue;
        var outSideValue = fixedValue + this.min;
        this.onValueChange(outSideValue);
        this.onChange.emit(outSideValue);
    };
    SliderComponent.prototype.writeValue = function (value) {
        var sideValue = value ? value - this.min : this.min;
        this.operateValue(sideValue);
        this.changeDetectorRef.markForCheck();
    };
    SliderComponent.prototype.registerOnChange = function (fn) {
        this.onValueChange = fn;
    };
    SliderComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SliderComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-slider',
                    template: "\n    <div class=\"slider-body\"><div class=\"slider-container\"><div #slider class=\"slider\"><div class=\"base-line\" (click)=\"onSliderClick($event)\"><div class=\"active-line\" [style.width.px]=\"handleOffset\"></div></div></div><div #sliderDot [style.left.px]=\"handleOffset\" class=\"handle\" [attr.title]=\"value\"></div></div><div class=\"slider-operator-container\"><button class=\"btn reduce\" (click)=\"onOperateValue(value - 1)\">-</button> <button class=\"btn plus\" (click)=\"onOperateValue(value + 1)\">+</button></div></div>\n  ",
                    styles: ["\n    :host{display:block;position:relative;overflow:hidden;width:100%;box-sizing:border-box}.slider-body{display:inline-block;position:relative;padding:0 45px;height:30px;width:100%}.slider-container{position:relative;display:inline-block;width:100%;height:100%;user-select:none;z-index:1}.slider{position:relative;height:100%;width:100%}.base-line{height:6px;width:100%;border-radius:3px;background-color:#ddd;position:absolute;left:0;top:50%;margin-top:-3px;overflow:hidden}.active-line{height:100%;width:0;background-color:#2196f3;position:relative;left:0}.handle{background-color:#2196f3;width:18px;height:18px;border-radius:50%;cursor:pointer;position:absolute;top:50%;margin-top:-9px;border:3px solid #fff;box-sizing:border-box;margin-left:-9px}.slider-operator-container{color:#2196f3}.slider-operator-container .btn{background:0 0;position:absolute;outline:0;box-shadow:none;font-size:30px;padding-left:20px;padding-right:20px}.slider-operator-container .btn:focus,.slider-operator-container .btn:hover{color:#2196f3}.slider-operator-container .btn.reduce{left:0;top:-18px}.slider-operator-container .btn.plus{right:0;top:-19px}\n  "],
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'slider',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return SliderComponent; }),
                            multi: true
                        }
                    ]
                },] },
    ];
    /** @nocollapse */
    SliderComponent.ctorParameters = function () { return [
        { type: ChangeDetectorRef, },
        { type: RebirthNGConfig, },
        { type: DocumentRef, },
        { type: WindowRef, },
    ]; };
    SliderComponent.propDecorators = {
        "dot": [{ type: ViewChild, args: ['sliderDot',] },],
        "slider": [{ type: ViewChild, args: ['slider',] },],
        "onChange": [{ type: Output },],
        "max": [{ type: Input },],
        "min": [{ type: Input },],
        "onResize": [{ type: HostListener, args: ['window:resize',] },],
    };
    return SliderComponent;
}());
export { SliderComponent };
//# sourceMappingURL=slider.component.js.map