import { Component, forwardRef, Input } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { RebirthNGConfig } from '../rebirth-ng.config';
var TagsComponent = /** @class */ (function () {
    function TagsComponent(rebirthNGConfig) {
        this.rebirthNGConfig = rebirthNGConfig;
        this.mutipleItems = [];
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.type = this.rebirthNGConfig.tags.type;
        this.newTagText = this.rebirthNGConfig.tags.newTagText;
        this.plusIcon = this.rebirthNGConfig.tags.plusIcon;
        this.removeIcon = this.rebirthNGConfig.tags.removeIcon;
        this.maxlength = this.rebirthNGConfig.tags.maxlength;
        this.maxSize = this.rebirthNGConfig.tags.maxSize;
    }
    TagsComponent.prototype.writeValue = function (obj) {
        this.mutipleItems = typeof obj === 'string' ? [obj] : (obj || []);
    };
    TagsComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    TagsComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    TagsComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    TagsComponent.prototype.newTag = function () {
        if (this.canAddMore()) {
            this.isTyping = true;
        }
    };
    TagsComponent.prototype.removeLabel = function (label) {
        this.mutipleItems = this.mutipleItems.filter(function (item) { return label !== item; });
        this.onTouched();
        this.onChange(this.mutipleItems);
    };
    TagsComponent.prototype.addLabel = function (label) {
        this.onTouched();
        if (label && this.mutipleItems.indexOf(label) === -1 && this.canAddMore()) {
            this.mutipleItems.push(label);
            this.onChange(this.mutipleItems);
        }
        this.selectValue = '';
        this.isTyping = false;
    };
    TagsComponent.prototype.canAddMore = function () {
        return !this.maxSize || this.mutipleItems.length < this.maxSize;
    };
    TagsComponent.prototype.boxBlur = function () {
        this.isTyping = false;
    };
    TagsComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-tags',
                    template: "\n    <ul class=\"tags {{cssClass}}\"><li *ngFor=\"let item of mutipleItems\" class=\"tag-item\"><span class=\"label label-{{type}} tag\" [ngClass]=\"{disabled: disabled}\">{{item}}<i aria-hidden=\"true\" class=\"label-btn label-remove {{removeIcon}}\" *ngIf=\"!disabled\" (click)=\"removeLabel(item)\"></i></span></li><li *ngIf=\"!disabled && !isTyping\" (click)=\"newTag()\" class=\"new-tag-item\"><button class=\"btn label label-{{type}} btn-new-tag\" [disabled]=\"!canAddMore()\">{{newTagText}}<i aria-hidden=\"true\" class=\"label-btn label-plus {{plusIcon}}\"></i></button></li><li *ngIf=\"!disabled && isTyping\" class=\"tag-input-panel\"><input [reAutoFocus]=\"isTyping\" name=\"tagsInput\" class=\"form-control tag-input-control\" [maxlength]=\"maxlength\" [(ngModel)]=\"selectValue\" (keydown.enter)=\"addLabel(selectValue)\" (blur)=\"boxBlur()\"></li></ul>\n  ",
                    styles: ["\n    .tags{display:block;list-style:none;width:100%;padding:0}.tags li{display:inline-block;margin:8px 8px 0 0}.tags li .label-btn{cursor:pointer;margin-left:8px}.tags .tag-input-control{max-width:10em}\n  "],
                    exportAs: 'tags',
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return TagsComponent; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    TagsComponent.ctorParameters = function () { return [
        { type: RebirthNGConfig, },
    ]; };
    TagsComponent.propDecorators = {
        "cssClass": [{ type: Input },],
        "newTagText": [{ type: Input },],
        "plusIcon": [{ type: Input },],
        "removeIcon": [{ type: Input },],
        "maxlength": [{ type: Input },],
        "maxSize": [{ type: Input },],
        "type": [{ type: Input },],
    };
    return TagsComponent;
}());
export { TagsComponent };
//# sourceMappingURL=tags.component.js.map