import { Component, Input, TemplateRef, ViewChild, ElementRef, Renderer2 } from '@angular/core';
import { TreeViewComponent } from './tree-view.component';
import { DraggableDirective } from '../draggable';
import { TreePanelComponent } from './tree-panel.component';
import { of } from 'rxjs/observable/of';
import 'rxjs/add/operator/map';
var TreeNodeComponent = /** @class */ (function () {
    function TreeNodeComponent(treeViewComponent, treePanelComponent, renderer) {
        var _this = this;
        this.treeViewComponent = treeViewComponent;
        this.treePanelComponent = treePanelComponent;
        this.renderer = renderer;
        this.checkable = false;
        this.lazyLoad = false;
        this.allowDraggable = false;
        this.allowMutipleSelected = false;
        this.isAcceptDrop = function ($event) {
            var dropData = JSON.parse($event.dataTransfer.getData(DraggableDirective.DRAGGABLE_DATA_KEY));
            if (!dropData.data.node) {
                return false;
            }
            if (_this.isNodeMyself(dropData)) {
                return false;
            }
            if (_this.isDescendant(dropData.data.node, _this.node)) {
                return false;
            }
            return true;
        };
    }
    TreeNodeComponent.prototype.getNodeExpendIcon = function () {
        if (this.isLeaf()) {
            return this.leafIcon;
        }
        return this.node.$expend ? this.expendIcon : this.collapseIcon;
    };
    TreeNodeComponent.prototype.onExpendedIconClick = function ($event) {
        var _this = this;
        $event.stopPropagation();
        if (this.isLeaf() || this.isLoading) {
            return;
        }
        var loadObservable = of(null);
        if (this.lazyLoad && !this.node.$loaded) {
            this.isLoading = true;
            loadObservable = this.loadChildren(this.node)
                .map(function (children) {
                _this.node.children = children || [];
                _this.node.$loaded = true;
            });
        }
        loadObservable.subscribe(function () {
            _this.isLoading = false;
            _this.node.$expend = !_this.node.$expend;
            _this.treeViewComponent.onNodeItemExpended(_this.node);
        }, function () {
            _this.isLoading = false;
        });
    };
    TreeNodeComponent.prototype.onNodeItemClick = function ($event) {
        $event.stopPropagation();
        this.node.$select = !this.node.$select;
        this.treeViewComponent.onNodeItemClicked(this.node);
    };
    TreeNodeComponent.prototype.onNodeItemDbClicked = function () {
        this.treeViewComponent.onNodeItemDbClicked(this.node);
    };
    TreeNodeComponent.prototype.nodeItemCheckedChange = function (checked) {
        this.changeChildrenChecked(this.node.children, checked);
        this.treePanelComponent.onNodeItemCheckedChanged(this.node);
    };
    TreeNodeComponent.prototype.onChildrenNodeCheckedChange = function (node) {
        this.node.$check = !this.node.children.some(function (item) { return !item.$check; });
        this.treePanelComponent.onNodeItemCheckedChanged(node);
    };
    TreeNodeComponent.prototype.isLeaf = function () {
        if (this.lazyLoad && !this.node.$loaded) {
            return false;
        }
        return !this.node.children || !this.node.children.length;
    };
    TreeNodeComponent.prototype.onDragEnter = function () {
        this.renderer.addClass(this.nodeItemContent.nativeElement, 'drop-node-enter');
    };
    TreeNodeComponent.prototype.onDragLeave = function () {
        var _this = this;
        setTimeout(function () {
            _this.renderer.removeClass(_this.nodeItemContent.nativeElement, 'drop-node-enter');
        });
    };
    TreeNodeComponent.prototype.onDropNodeItem = function ($event) {
        var _this = this;
        var dropData = JSON.parse($event.dataTransfer.getData(DraggableDirective.DRAGGABLE_DATA_KEY));
        if (dropData.data.node && !this.isNodeMyself(dropData)) {
            this.treeViewComponent.onNodeItemDroped({ target: this.node, data: dropData });
        }
        setTimeout(function () {
            _this.renderer.removeClass(_this.nodeItemContent.nativeElement, 'drop-node-enter');
        }, 0);
    };
    TreeNodeComponent.prototype.isNodeMyself = function (dropData) {
        return this.node[this.valueField] === dropData.data.node[this.valueField];
    };
    TreeNodeComponent.prototype.isDescendant = function (parent, target) {
        var _this = this;
        if (!parent) {
            return false;
        }
        return (parent.children || []).some(function (nodeItem) {
            return nodeItem[_this.valueField] === target[_this.valueField] || _this.isDescendant(nodeItem, target);
        });
    };
    TreeNodeComponent.prototype.changeChildrenChecked = function (nodes, checked) {
        var _this = this;
        if (!nodes) {
            return;
        }
        nodes.forEach(function (node) {
            node.$check = checked;
            _this.changeChildrenChecked(node.children, checked);
        });
    };
    TreeNodeComponent.decorators = [
        { type: Component, args: [{
                    selector: 're-tree-node,[reTreeNode]',
                    template: "\n    <div class=\"node-item {{nodeCssClass}}\" reDraggable=\"re:node-item-drop\" [disabled]=\"!allowDraggable\" [dragData]=\"{node: node, parent: parentNode}\"><div class=\"node-item-panel\"><i class=\"node-expend-icon {{getNodeExpendIcon()}}\" (click)=\"onExpendedIconClick($event)\"></i><div class=\"node-item-content\" #nodeItemContent reDroppable=\"re:node-item-drop\" (onDrop)=\"onDropNodeItem($event)\" (onDragEnter)=\"onDragEnter()\" (onDragLeave)=\"onDragLeave()\" [acceptDrop]=\"isAcceptDrop\"><div class=\"node-check-box\"><re-checkbox *ngIf=\"checkable\" [(ngModel)]=\"node.$check\" label=\"\" (valueChange)=\"nodeItemCheckedChange($event)\"></re-checkbox></div><ng-container *ngIf=\"!nodeItemTemplate\"><span class=\"node-text\" [ngClass]=\"{'bg-primary': node.$select}\" (click)=\"onNodeItemClick($event)\" (dblclick)=\"onNodeItemDbClicked($event)\"><i *ngIf=\"iconField && node[iconField]\" class=\"node-icon {{node[iconField]}}\"></i> {{node[textField] || ''}} <span class=\"loading\" *ngIf=\"isLoading\"><i class=\"{{loadingIcon}}\"></i></span></span><div class=\"toolbar\"><ng-template *ngIf=\"nodeItemToolbarTemplate\" [ngTemplateOutlet]=\"nodeItemToolbarTemplate\" [ngTemplateOutletContext]=\"{$implicit: node, parentNode:parentNode, tree: this}\"></ng-template></div></ng-container><ng-container *ngIf=\"nodeItemTemplate\"><ng-template *ngIf=\"nodeItemTemplate\" [ngTemplateOutlet]=\"nodeItemTemplate\" [ngTemplateOutletContext]=\"{$implicit: node,  parentNode:parentNode, tree: this }\"></ng-template></ng-container></div></div><re-tree-panel *ngIf=\"node.children && node.children.length && node.$expend\" class=\"children-view\" [treeData]=\"node.children\" [textField]=\"textField\" [valueField]=\"valueField\" [iconField]=\"iconField\" [checkable]=\"checkable\" [lazyLoad]=\"lazyLoad\" [loadingIcon]=\"loadingIcon\" [loadChildren]=\"loadChildren\" [allowDraggable]=\"allowDraggable && !isInDraggle\" [allowMutipleSelected]=\"allowMutipleSelected\" [parentNode]=\"node\" [nodeItemTemplate]=\"nodeItemTemplate\" [nodeItemToolbarTemplate]=\"nodeItemToolbarTemplate\" [leafIcon]=\"leafIcon\" [expendIcon]=\"expendIcon\" [collapseIcon]=\"collapseIcon\" (nodeItemCheckedChanged)=\"onChildrenNodeCheckedChange($event)\"></re-tree-panel></div>\n  ",
                    styles: ["\n    .node-item{position:relative}.node-item-content ::ng-deep{display:inline-block}.node-item-content ::ng-deep .node-icon{margin-right:2px}.node-item-content ::ng-deep .node-text{cursor:pointer;padding:2px;border-radius:4px}.node-item-content ::ng-deep .node-text:not(.bg-primary):hover{border:1px dashed #ddd}.node-expend-icon{cursor:pointer}.drop-node-enter{border:1px solid #2196f3}.node-check-box,.toolbar{display:inline-block}.expend-icon{cursor:pointer}.node-check-box .checkbox{margin:0}.children-view{display:block;padding:10px 1.5rem;border-left:1px dashed #ddd;margin-left:5px}.children-view .node-item{position:relative}.children-view .node-item:before{content:' ';position:absolute;left:-1.5rem;border-top:1px dashed #ddd;display:inline-block;width:1.5rem;height:1px;top:10px}.loading{-webkit-animation:loadingAnimation 1.5s infinite alternate;-moz-animation:loadingAnimation 1.5s infinite alternate;-ms-animation:loadingAnimation 1.5s infinite alternate;-o-animation:loadingAnimation 1.5s infinite alternate;animation:loadingAnimation 1.5s infinite alternate}@-webkit-keyframes loadingAnimation{50%{color:#ff9800}100%{color:#4caf50}}\n  "],
                    exportAs: 'treeNode'
                },] },
    ];
    /** @nocollapse */
    TreeNodeComponent.ctorParameters = function () { return [
        { type: TreeViewComponent, },
        { type: TreePanelComponent, },
        { type: Renderer2, },
    ]; };
    TreeNodeComponent.propDecorators = {
        "node": [{ type: Input },],
        "parentNode": [{ type: Input },],
        "valueField": [{ type: Input },],
        "textField": [{ type: Input },],
        "nodeCssClass": [{ type: Input },],
        "iconField": [{ type: Input },],
        "checkable": [{ type: Input },],
        "lazyLoad": [{ type: Input },],
        "loadingIcon": [{ type: Input },],
        "loadChildren": [{ type: Input },],
        "allowDraggable": [{ type: Input },],
        "allowMutipleSelected": [{ type: Input },],
        "nodeItemTemplate": [{ type: Input },],
        "nodeItemToolbarTemplate": [{ type: Input },],
        "leafIcon": [{ type: Input },],
        "expendIcon": [{ type: Input },],
        "collapseIcon": [{ type: Input },],
        "nodeItemContent": [{ type: ViewChild, args: ['nodeItemContent',] },],
    };
    return TreeNodeComponent;
}());
export { TreeNodeComponent };
//# sourceMappingURL=tree-node.component.js.map